// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.


// Code generated from the elasticsearch-specification DO NOT EDIT.
// https://github.com/elastic/elasticsearch-specification/tree/4316fc1aa18bb04678b156f23b22c9d3f996f9c9


// Returns information about whether a particular component template exist
package existscomponenttemplate

import (
	gobytes "bytes"
	"context"
	"errors"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"net/url"
	"strconv"
	"strings"

	"github.com/elastic/elastic-transport-go/v8/elastictransport"
)

const (
	nameMask = iota + 1
)

// ErrBuildPath is returned in case of missing parameters within the build of the request.
var ErrBuildPath = errors.New("cannot build path, check for missing path parameters")

type ExistsComponentTemplate struct {
	transport elastictransport.Interface

	headers http.Header
	values  url.Values
	path    url.URL

	buf *gobytes.Buffer

	paramSet int

	name string
}

// NewExistsComponentTemplate type alias for index.
type NewExistsComponentTemplate func(name string) *ExistsComponentTemplate

// NewExistsComponentTemplateFunc returns a new instance of ExistsComponentTemplate with the provided transport.
// Used in the index of the library this allows to retrieve every apis in once place.
func NewExistsComponentTemplateFunc(tp elastictransport.Interface) NewExistsComponentTemplate {
	return func(name string) *ExistsComponentTemplate {
		n := New(tp)

		n.Name(name)

		return n
	}
}

// Returns information about whether a particular component template exist
//
// https://www.elastic.co/guide/en/elasticsearch/reference/{branch}/indices-component-template.html
func New(tp elastictransport.Interface) *ExistsComponentTemplate {
	r := &ExistsComponentTemplate{
		transport: tp,
		values:    make(url.Values),
		headers:   make(http.Header),
		buf:       gobytes.NewBuffer(nil),
	}

	return r
}

// HttpRequest returns the http.Request object built from the
// given parameters.
func (r *ExistsComponentTemplate) HttpRequest(ctx context.Context) (*http.Request, error) {
	var path strings.Builder
	var method string
	var req *http.Request

	var err error

	r.path.Scheme = "http"

	switch {
	case r.paramSet == nameMask:
		path.WriteString("/")
		path.WriteString("_component_template")
		path.WriteString("/")
		path.WriteString(url.PathEscape(r.name))

		method = http.MethodHead
	}

	r.path.Path = path.String()
	r.path.RawQuery = r.values.Encode()

	if r.path.Path == "" {
		return nil, ErrBuildPath
	}

	if ctx != nil {
		req, err = http.NewRequestWithContext(ctx, method, r.path.String(), r.buf)
	} else {
		req, err = http.NewRequest(method, r.path.String(), r.buf)
	}

	req.Header.Set("accept", "application/vnd.elasticsearch+json;compatible-with=8")

	if err != nil {
		return req, fmt.Errorf("could not build http.Request: %w", err)
	}

	return req, nil
}

// Do runs the http.Request through the provided transport.
func (r ExistsComponentTemplate) Do(ctx context.Context) (*http.Response, error) {
	req, err := r.HttpRequest(ctx)
	if err != nil {
		return nil, err
	}

	res, err := r.transport.Perform(req)
	if err != nil {
		return nil, fmt.Errorf("an error happened during the ExistsComponentTemplate query execution: %w", err)
	}

	return res, nil
}

// IsSuccess allows to run a query with a context and retrieve the result as a boolean.
// This only exists for endpoints without a request payload and allows for quick control flow.
func (r ExistsComponentTemplate) IsSuccess(ctx context.Context) (bool, error) {
	res, err := r.Do(ctx)

	if err != nil {
		return false, err
	}
	io.Copy(ioutil.Discard, res.Body)
	err = res.Body.Close()
	if err != nil {
		return false, err
	}

	if res.StatusCode >= 200 && res.StatusCode < 300 {
		return true, nil
	}

	return false, nil
}

// Header set a key, value pair in the ExistsComponentTemplate headers map.
func (r *ExistsComponentTemplate) Header(key, value string) *ExistsComponentTemplate {
	r.headers.Set(key, value)

	return r
}

// Name Comma-separated list of component template names used to limit the request.
// Wildcard (*) expressions are supported.
// API Name: name
func (r *ExistsComponentTemplate) Name(v string) *ExistsComponentTemplate {
	r.paramSet |= nameMask
	r.name = v

	return r
}

// MasterTimeout Period to wait for a connection to the master node. If no response is
// received before the timeout expires, the request fails and returns an
// error.
// API name: master_timeout
func (r *ExistsComponentTemplate) MasterTimeout(value string) *ExistsComponentTemplate {
	r.values.Set("master_timeout", value)

	return r
}

// Local If true, the request retrieves information from the local node only.
// Defaults to false, which means information is retrieved from the master node.
// API name: local
func (r *ExistsComponentTemplate) Local(b bool) *ExistsComponentTemplate {
	r.values.Set("local", strconv.FormatBool(b))

	return r
}
