// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.


// Code generated from the elasticsearch-specification DO NOT EDIT.
// https://github.com/elastic/elasticsearch-specification/tree/4316fc1aa18bb04678b156f23b22c9d3f996f9c9


// Mount a snapshot as a searchable index.
package mount

import (
	gobytes "bytes"
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"net/http"
	"net/url"
	"strconv"
	"strings"

	"github.com/elastic/elastic-transport-go/v8/elastictransport"
)

const (
	repositoryMask = iota + 1

	snapshotMask
)

// ErrBuildPath is returned in case of missing parameters within the build of the request.
var ErrBuildPath = errors.New("cannot build path, check for missing path parameters")

type Mount struct {
	transport elastictransport.Interface

	headers http.Header
	values  url.Values
	path    url.URL

	buf *gobytes.Buffer

	req *Request
	raw json.RawMessage

	paramSet int

	repository string
	snapshot   string
}

// NewMount type alias for index.
type NewMount func(repository, snapshot string) *Mount

// NewMountFunc returns a new instance of Mount with the provided transport.
// Used in the index of the library this allows to retrieve every apis in once place.
func NewMountFunc(tp elastictransport.Interface) NewMount {
	return func(repository, snapshot string) *Mount {
		n := New(tp)

		n.Repository(repository)

		n.Snapshot(snapshot)

		return n
	}
}

// Mount a snapshot as a searchable index.
//
// https://www.elastic.co/guide/en/elasticsearch/reference/master/searchable-snapshots-api-mount-snapshot.html
func New(tp elastictransport.Interface) *Mount {
	r := &Mount{
		transport: tp,
		values:    make(url.Values),
		headers:   make(http.Header),
		buf:       gobytes.NewBuffer(nil),
	}

	return r
}

// Raw takes a json payload as input which is then passed to the http.Request
// If specified Raw takes precedence on Request method.
func (r *Mount) Raw(raw json.RawMessage) *Mount {
	r.raw = raw

	return r
}

// Request allows to set the request property with the appropriate payload.
func (r *Mount) Request(req *Request) *Mount {
	r.req = req

	return r
}

// HttpRequest returns the http.Request object built from the
// given parameters.
func (r *Mount) HttpRequest(ctx context.Context) (*http.Request, error) {
	var path strings.Builder
	var method string
	var req *http.Request

	var err error

	if r.raw != nil {
		r.buf.Write(r.raw)
	} else if r.req != nil {
		data, err := json.Marshal(r.req)

		if err != nil {
			return nil, fmt.Errorf("could not serialise request for Mount: %w", err)
		}

		r.buf.Write(data)
	}

	r.path.Scheme = "http"

	switch {
	case r.paramSet == repositoryMask|snapshotMask:
		path.WriteString("/")
		path.WriteString("_snapshot")
		path.WriteString("/")
		path.WriteString(url.PathEscape(r.repository))
		path.WriteString("/")
		path.WriteString(url.PathEscape(r.snapshot))
		path.WriteString("/")
		path.WriteString("_mount")

		method = http.MethodPost
	}

	r.path.Path = path.String()
	r.path.RawQuery = r.values.Encode()

	if r.path.Path == "" {
		return nil, ErrBuildPath
	}

	if ctx != nil {
		req, err = http.NewRequestWithContext(ctx, method, r.path.String(), r.buf)
	} else {
		req, err = http.NewRequest(method, r.path.String(), r.buf)
	}

	if r.buf.Len() > 0 {
		req.Header.Set("content-type", "application/vnd.elasticsearch+json;compatible-with=8")
	}

	req.Header.Set("accept", "application/vnd.elasticsearch+json;compatible-with=8")

	if err != nil {
		return req, fmt.Errorf("could not build http.Request: %w", err)
	}

	return req, nil
}

// Do runs the http.Request through the provided transport.
func (r Mount) Do(ctx context.Context) (*http.Response, error) {
	req, err := r.HttpRequest(ctx)
	if err != nil {
		return nil, err
	}

	res, err := r.transport.Perform(req)
	if err != nil {
		return nil, fmt.Errorf("an error happened during the Mount query execution: %w", err)
	}

	return res, nil
}

// Header set a key, value pair in the Mount headers map.
func (r *Mount) Header(key, value string) *Mount {
	r.headers.Set(key, value)

	return r
}

// Repository The name of the repository containing the snapshot of the index to mount
// API Name: repository
func (r *Mount) Repository(v string) *Mount {
	r.paramSet |= repositoryMask
	r.repository = v

	return r
}

// Snapshot The name of the snapshot of the index to mount
// API Name: snapshot
func (r *Mount) Snapshot(v string) *Mount {
	r.paramSet |= snapshotMask
	r.snapshot = v

	return r
}

// MasterTimeout Explicit operation timeout for connection to master node
// API name: master_timeout
func (r *Mount) MasterTimeout(value string) *Mount {
	r.values.Set("master_timeout", value)

	return r
}

// WaitForCompletion Should this request wait until the operation has completed before returning
// API name: wait_for_completion
func (r *Mount) WaitForCompletion(b bool) *Mount {
	r.values.Set("wait_for_completion", strconv.FormatBool(b))

	return r
}

// Storage Selects the kind of local storage used to accelerate searches. Experimental,
// and defaults to `full_copy`
// API name: storage
func (r *Mount) Storage(value string) *Mount {
	r.values.Set("storage", value)

	return r
}
